<!DOCTYPE html>
<html lang="en">
  <head>
    <title></title>
    <link rel="icon" type="image/x-icon" href="@root/static/favicon.ico">
    <script src="@root/static/slides.js" defer></script>
    <meta charset="UTF-8">
    <link rel="stylesheet" href="@root/static/slides.css">
  </head>
  <body>
{{content}}
  </body>
</html>
