<!DOCTYPE html>
<html>
  <head>
    <title></title>
    {% if icon_file %}
    <link rel="icon" type="image/x-icon" href="@root/{{icon_file}}">
    {% else %}
    <link rel="icon" type="image/x-icon" href="@root/static/favicon.ico">
    {% endif %}
    <meta charset="UTF-8">
    {% if css_file %}
    <link rel="stylesheet" href="@root/{{css_file}}" type="text/css">
    {% else %}
    <link rel="stylesheet" href="@root/static/page.css" type="text/css">
    {% endif %}
  </head>
  <body>
    <main>
{{content}}
    </main>
    <footer>
      Copyright &copy; 2024 <a href="@root/contributing.html">the authors</a>
    </footer>
  </body>
</html>
